import bpy
import os

from . import translations


# list prop
class SHYNEZ_ANIME_BGTimesheet_UL_PropGroup(bpy.types.PropertyGroup):
    frame: bpy.props.IntProperty(name="Frame", default=0)
    filepath: bpy.props.StringProperty(name="Filepath", default="")
    basename: bpy.props.StringProperty(name="Basename", default="")

# panel
class SHYNEZ_ANIME_BGTimesheetPanel(bpy.types.Panel):
    # shynez panel: BG Timesheet
    bl_label = "Bg Timesheet"
    bl_idname = "SHYNEZ_PT_bgtimesheet_panel"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "Shynez Anime"

    def draw(self, context):
        scene = context.scene
        camera = context.scene.camera
        layout = self.layout # layout
        
        if camera:
            row = layout.row(align=True)
            row.operator("shynez_bgtimesheet.initkeyframe") # create bgt index keyframe
            row.separator(factor=3.0)
            
            sub = row.row(align=True)
            sub.operator("shynez_bgtimesheet.remove") # remove single
            row.separator(factor=2.0)
        
            sub = row.row()
            sub.operator("shynez_bgtimesheet.clear", icon='TRASH') # list clear
        
            row = layout.row()
            row.template_list("SHYNEZ_UL_bgtimesheet_list", "BG Timesheet List", context.scene, "shynez_bgtimesheet", context.scene.camera.data, "shynez_bgtimesheet_active_index") # list
        
        row = layout.row()
        if camera and len(camera.data.background_images) > 0:
            bg = camera.data.background_images[0]
            row.prop(bg, "alpha")
            row = layout.row()
            row.prop(bg, "display_depth", expand=True)
        if camera:
            row = layout.row()
            row.prop(scene, "frame_current")
        row = layout.row()
        split = row.split(factor = 0.3)
        if camera and len(camera.data.background_images) > 0:
            bg = camera.data.background_images[0]
            split.prop(bg, "show_background_image", text="Show")
        else:
            split.label(text = "———")
            
        if camera:
            split.operator("shynez_bgtimesheet.add", text="Add")
        else:
            split.label(text = "No Camera")

class SHYNEZ_ANIME_BgtimesheetList(bpy.types.UIList):
    bl_idname = "SHYNEZ_UL_bgtimesheet_list"
    
    def draw_item(self, context, layout, data, item, icon, active_data, active_propname, index, flt_flag):
        split = layout.split(factor = 0.25)
        split.label(text = str(item.frame))
        split.label(text = item.basename)

class SHYNEZ_ANIME_BgtimesheetInitBgKeyframe(bpy.types.Operator):
    bl_label = "Reset Keyframes"
    bl_idname = "shynez_bgtimesheet.initkeyframe"
    bl_description = "Reset background image keyframes to active camera data."
    
    def execute(self, context):
        init_bgts_keyframes(self, context)
        return {'FINISHED'}

class SHYNEZ_ANIME_BgtimesheetClear(bpy.types.Operator):
    bl_label = ""
    bl_idname = "shynez_bgtimesheet.clear"
    bl_description = "Clear All."

    def execute(self, context):
        bpy.ops.wm.call_menu(name=SHYNEZ_ANIME_BgtimesheetClearConfirmMenu.bl_idname) # confirm menu
        return {'FINISHED'}
    
class SHYNEZ_ANIME_BgtimesheetClearConfirmMenu(bpy.types.Menu):
    bl_label = "Clear List"
    bl_idname = "SHYNEZ_MT_bgtimesheet_clearconfirmMenu"
    
    def draw(self, context):
        layout = self.layout
        layout.operator("shynez_bgtimesheet.clearconfirm")

class SHYNEZ_ANIME_BgtimesheetClearConfirm(bpy.types.Operator):
    bl_label = "Confirm"
    bl_idname = "shynez_bgtimesheet.clearconfirm"
    bl_description = "Confirm CLEAR."
    
    def execute(self, context):
        context.scene.shynez_bgtimesheet.clear() # clear list
        if context.scene.camera:
            context.scene.camera.data.background_images.clear()
            context.scene.camera.data.background_images.update()
        init_bgts_keyframes(self, context)
        return {'FINISHED'}
    
class SHYNEZ_ANIME_BgtimesheetAdd(bpy.types.Operator):
    bl_label = "Add Image"
    bl_idname = "shynez_bgtimesheet.add"
    bl_description = "Add an image, replace if already exist at current frame."
    
    filepath: bpy.props.StringProperty(subtype="FILE_PATH")
    filter_glob: bpy.props.StringProperty( default='*.jpg;*.jpeg;*.png;*.tif;*.tiff;*.bmp;*.tga', options={'HIDDEN'} )

    def execute(self, context):
        if not context.scene.camera:
            self.report({'WARNING'}, "No Camera")
            return {'CANCELLED'}
        
        image_extensions = ['.jpg', '.jpeg', '.png', '.tif', 'tiff', '.bmp', '.tga']
        _, ext = os.path.splitext(self.filepath.lower())
        if ext in image_extensions:
            # append new item to data list
            addFlag = False
            for item in context.scene.shynez_bgtimesheet:
                if item.frame == context.scene.frame_current:
                    item.filepath = self.filepath
                    item.basename = bpy.path.display_name(self.filepath, has_ext=True)
                    addFlag = True
                    break
            if addFlag == False:
                new_item = context.scene.shynez_bgtimesheet.add()
                new_item.frame = context.scene.frame_current
                new_item.filepath = self.filepath
                new_item.basename = bpy.path.display_name(self.filepath, has_ext=True)
                
            # open image
            bpy.ops.image.open(filepath=self.filepath, filter_image=True)
            # get image
            for img in bpy.data.images:
                if img.filepath == self.filepath:
                    # get image size
                    width, height = img.size
                    # update render resolution
                    bpy.context.scene.render.resolution_x = width
                    bpy.context.scene.render.resolution_y = height
                    
                    set_bgts_background_image(self, img)
                    break
                
            # sort
            sorted_list = sorted(context.scene.shynez_bgtimesheet, key=lambda obj: obj.get("frame"))

            # append sorted data
            for index in range(len(sorted_list)):
                #print(sorted_list[index].frame, sorted_list[index].basename, sorted_list[index].filepath)
                
                new_item = context.scene.shynez_bgtimesheet.add()
                new_item.frame = sorted_list[index].frame
                new_item.filepath = sorted_list[index].filepath
                new_item.basename = sorted_list[index].basename
                
                # insert keyframe
                afterIndex = len(context.scene.shynez_bgtimesheet)-1 - len(sorted_list)
                print("afterIndex: ", afterIndex)
                if context.scene.camera:
                    context.scene.camera.data.shynez_bgtimesheet_active_index = afterIndex
                    context.scene.frame_current = new_item.frame
                    cam_animation_data = context.scene.camera.data.animation_data
                    if cam_animation_data: 
                        bgt_index_fcurve = cam_animation_data.action.fcurves.find('["shynez_bgtimesheet_active_index"]')
                        if bgt_index_fcurve: 
                            bgt_index_fcurve.lock = False
                    context.scene.camera.data.keyframe_insert(data_path='["shynez_bgtimesheet_active_index"]')
            
            # clear unsorted data
            for index in range(len(sorted_list)):
                context.scene.shynez_bgtimesheet.remove(0)
                
            init_bgts_keyframes(self, context)
        else:
            self.report({'WARNING'}, "Select An Image Please")
        return {'FINISHED'}

    def invoke(self, context, event):
        filepath = context.window_manager.fileselect_add(self)
        return {'RUNNING_MODAL'}
    
class SHYNEZ_ANIME_BgtimesheetRemove(bpy.types.Operator):
    bl_label = "Remove"
    bl_idname = "shynez_bgtimesheet.remove"
    bl_description = "Remove single active item."
    
    def execute(self, context):
        # remove active item
        my_list = context.scene.shynez_bgtimesheet
        if len(my_list):
            my_list.remove(context.scene.camera.data.shynez_bgtimesheet_active_index)
            if len(my_list) - 1 < context.scene.camera.data.shynez_bgtimesheet_active_index:
                context.scene.camera.data.shynez_bgtimesheet_active_index = len(my_list) - 1
            if context.scene.camera.data.shynez_bgtimesheet_active_index < 0:
                context.scene.camera.data.shynez_bgtimesheet_active_index = 0
                
            init_bgts_keyframes(self, context)
        return {'FINISHED'}

def init_bgts_keyframes(self, context):
    anim = context.scene.camera.data.animation_data
    fcurves = None
    if anim is not None and anim.action is not None:
        fcurve = anim.action.fcurves.find('["shynez_bgtimesheet_active_index"]')
        if fcurve:
            fcurve.keyframe_points.clear()
            fcurve.keyframe_points.update()
    for index in range(len(context.scene.shynez_bgtimesheet)):
        # insert keyframe
        if context.scene.camera:
            context.scene.camera.data.shynez_bgtimesheet_active_index = index
            context.scene.frame_current = context.scene.shynez_bgtimesheet[index].frame
            context.scene.camera.data.keyframe_insert(data_path='["shynez_bgtimesheet_active_index"]')
        else:
            self.report({'WARNING'}, "No Camera")
    if anim is not None and anim.action is not None and fcurves is not None and fcurves.keyframe_points is not None:
        for pt in fcurves.keyframe_points:
            pt.interpolation = 'CONSTANT'
            
    if context.scene.camera and len(context.scene.shynez_bgtimesheet) == 0:
        context.scene.camera.data.background_images.clear()
        context.scene.camera.data.background_images.update()

# set background image
def set_bgts_background_image(self, img):
    camera = bpy.context.scene.camera
    if camera:
        camera.data.show_background_images = True
        if len(camera.data.background_images) == 0:
            camera.data.background_images.new()
            camera.data.background_images[0].alpha = 0.15
            camera.data.background_images[0].display_depth = 'FRONT'
        camera.data.background_images[0].image = img
        camera.data.background_images[0].frame_method = 'FIT'
    else:
        self.report({'WARNING'}, "No Camera")

def on_bgts_background_image_change(self, context):
    if len(context.scene.shynez_bgtimesheet) == 0:
        context.scene.frame_current = 0
    else:
        context.scene.frame_current = context.scene.shynez_bgtimesheet[context.scene.camera.data.shynez_bgtimesheet_active_index].frame

# update background image
def update_bgts_background_image(self, context):
    if context.scene.shynez_bgtimesheet and len(context.scene.shynez_bgtimesheet) > 0:
        filepath = bpy.context.scene.shynez_bgtimesheet[context.scene.camera.data.shynez_bgtimesheet_active_index]['filepath']
        
        findFlag = False
        # get image
        for img in bpy.data.images:
            if img.filepath == filepath:
                findFlag = True
                set_bgts_background_image(self, img)
                break
        if not findFlag:
            for item in bpy.context.scene.shynez_bgtimesheet:
                bpy.ops.image.open(filepath=item.filepath, filter_image=True)
            # get image
            for img in bpy.data.images:
                if img.filepath == filepath:
                    findFlag = True
                    set_bgts_background_image(self, img)
                    break


classes = [
    SHYNEZ_ANIME_BGTimesheet_UL_PropGroup,
    SHYNEZ_ANIME_BGTimesheetPanel,
    SHYNEZ_ANIME_BgtimesheetInitBgKeyframe,
    SHYNEZ_ANIME_BgtimesheetClear,
    SHYNEZ_ANIME_BgtimesheetClearConfirmMenu,
    SHYNEZ_ANIME_BgtimesheetClearConfirm,
    SHYNEZ_ANIME_BgtimesheetAdd,
    SHYNEZ_ANIME_BgtimesheetRemove,
    SHYNEZ_ANIME_BgtimesheetList
]

class_register, class_unregister = bpy.utils.register_classes_factory(classes)


def register():
    bpy.app.translations.register(__name__, translations.translations_dict)
    
    class_register()
    bpy.types.Scene.shynez_bgtimesheet = bpy.props.CollectionProperty(type=SHYNEZ_ANIME_BGTimesheet_UL_PropGroup)
    bpy.types.Camera.shynez_bgtimesheet_active_index = bpy.props.IntProperty(name="Active Item",update=on_bgts_background_image_change)
    
    
    bpy.app.handlers.frame_change_post.append(update_bgts_background_image)
    bpy.app.handlers.persistent(update_bgts_background_image)


def unregister():
    class_unregister()
    del bpy.types.Scene.shynez_bgtimesheet
    del bpy.types.Camera.shynez_bgtimesheet_active_index
    
    bpy.app.handlers.frame_change_post.remove(update_bgts_background_image)
    
    bpy.app.translations.unregister(__name__)


if __name__ == "__main__":
    register()